// cubeview.cpp : implementation of the CCubeView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.


#include "stdafx.h"
#include "cube.h"



#pragma warning( once : 4305 4244 )        // Issue warning 4385
                                      //  only once.

#define M_PI 3.1415926535897932384626
#define NUM_BUBBLE_FRAMES 8
#define NUM_FADE_FRAMES 13


#include "cubedoc.h"
#include "cubeview.h"
#include "math.h"



#include "gl\gl.h"
#include "gl\glu.h"
#include "glut.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif




unsigned char threeto8[8] =
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] =
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] =
{
	0, 255
};

static int defaultOverride[13] =
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

static PALETTEENTRY defaultPalEntry[20] =
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};

/////////////////////////////////////////////////////////////////////////////
// CCubeView

IMPLEMENT_DYNCREATE(CCubeView, CView)

BEGIN_MESSAGE_MAP(CCubeView, CView)
	//{{AFX_MSG_MAP(CCubeView)
	ON_COMMAND(ID_FILE_PLAY, OnFilePlay)
	ON_UPDATE_COMMAND_UI(ID_FILE_PLAY, OnUpdateFilePlay)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_UPDATE_COMMAND_UI(ID_APP_CALIBRATE, OnUpdateAppCalibrate)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_KEYDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCubeView construction/destruction

CCubeView::CCubeView() 

{
	m_pDC = NULL;
	m_pOldPalette = NULL;
	m_play = FALSE;
}

CCubeView::~CCubeView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CCubeView drawing

void CCubeView::OnDraw(CDC* pDC)
{
	CCubeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	DrawScene();
}

/////////////////////////////////////////////////////////////////////////////
// CCubeView diagnostics

#ifdef _DEBUG
void CCubeView::AssertValid() const
{
	CView::AssertValid();
}

void CCubeView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CCubeDoc* CCubeView::GetDocument() // non-debug version is inline
{
	return STATIC_DOWNCAST(CCubeDoc, m_pDocument);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCubeView message handlers

void CCubeView::OnFilePlay()
{

}

void CCubeView::OnUpdateFilePlay(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_play);
}

BOOL CCubeView::PreCreateWindow(CREATESTRUCT& cs)
{
	// An OpenGL window must be created with the following flags and must not
	// include CS_PARENTDC for the class style. Refer to SetPixelFormat
	// documentation in the "Comments" section for further information.
	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

int CCubeView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	Init(); // initialize OpenGL
	GetDocument()->pview = this;

	return 0;
}

void CCubeView::OnDestroy()
{
	HGLRC   hrc;

	KillTimer(1);

	hrc = ::wglGetCurrentContext();

	::wglMakeCurrent(NULL,  NULL);

	if (hrc)
		::wglDeleteContext(hrc);

	if (m_pOldPalette)
		m_pDC->SelectPalette(m_pOldPalette, FALSE);

	if (m_pDC)
		delete m_pDC;

	CView::OnDestroy();
}

void CCubeView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);

	if(cy > 0)
	{
		glViewport(0, 0, cx, cy);

		if((m_oldRect.right > cx) || (m_oldRect.bottom > cy))
			RedrawWindow();

		m_oldRect.right = cx;
		m_oldRect.bottom = cy;



	//	glViewport((m_oldRect.Width()-m_oldRect.Height())/2,0,m_oldRect.Height(), m_oldRect.Height());
		glViewport(0,0,m_oldRect.Width(), m_oldRect.Height());
		fAspect = (double)m_oldRect.Width()/(double)m_oldRect.Height();

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(-fAspect*1.1,fAspect*1.1,-1.1,1.1,-1.1,1.1);



	//	SetCursorPos(m_oldRect.CenterPoint().x,m_oldRect.CenterPoint().y + 164);
		win_cent = Point(m_oldRect.CenterPoint().x,m_oldRect.CenterPoint().y + 164);


	}
}

void CCubeView::OnTimer(UINT nIDEvent)
{

	if (make_bubble && bubble_frame < NUM_BUBBLE_FRAMES)
		bubble_frame++;
	if (fade_target != -1 && fade_frame < NUM_FADE_FRAMES)
		fade_frame++;


	
	
	
	
///////////////////////////	
	if (!freeze)
	{
	CPoint point;
	GetCursorPos(&point);
	double current_time = GetSeconds();
	int sm = 0;
	if (point.x < win_cent.x - 1)
		sm = 1;
	else if (point.x > win_cent.x + 1)
		sm = 1;

	if (point.y < win_cent.y - 1)
		sm = 1;
	else if (point.y > win_cent.y + 1)
		sm = 1;




	Point d;
	d.x = point.x - old_mouse_loc.x;
	d.y = point.y - old_mouse_loc.y;
	mouse_loc.x = mouse_loc.x+ d.x;
	mouse_loc.y = mouse_loc.y+ d.y;




	old_mouse_loc.x = point.x;
	old_mouse_loc.y = point.y;
	point.x = mouse_loc.x;
	point.y = mouse_loc.y;






	


	GLdouble objx, objy, objz;

	CRect rect;
    GetClientRect(&rect);
    int y = rect.Height() - point.y;
	GLint viewport[4];
	GLdouble modelMatrix[16], projMatrix[16], winz;
	glPushMatrix();


	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
	glGetIntegerv(GL_VIEWPORT, viewport);
	

	winz = 0;

	gluUnProject(point.x, y, winz, modelMatrix, projMatrix, viewport, &objx, &objy, &objz);	
	Point old = cursor_loc;
	cursor_loc = Point(objx,objy);
	d = cursor_loc - old;

	if (flatten_tim)
		cursor_loc.y = 0;
//TRACE("timer: %2.2f,%2.2f\n",cursor_loc.x,cursor_loc.y);

	tim_loc.x = tim_loc.x+ d.x;
	tim_loc.y = tim_loc.y+ d.y;

//	TRACE("timer: %2.2f,%2.2f\n",tim_loc.x,tim_loc.y);

	
	for (int i = 0; i < NUM_TIM_POINTS - 1; i++)
	{
		mouse_points[i] = mouse_points[i+1];
		tim_points[i] = tim_points[i+1];
		tim_point_times[i] = tim_point_times[i+1];
	}


	mouse_points[NUM_TIM_POINTS-1] = mouse_loc;
	tim_points[NUM_TIM_POINTS-1] = cursor_loc;
	tim_point_times[NUM_TIM_POINTS-1] = current_time;
	

	CalcCurrentTarget();



//	TRACE("%d, %d\n",point.x, point.y);
	if (sm && !freeze)
	{
		SetCursorPos(win_cent.x,win_cent.y);
		old_mouse_loc = win_cent;
	}
	}
///////////////////////////	
	
	
	double now_time = GetSeconds();


	if (trial_start_time == -1)
	{
		if ((cursor_loc - delay_start_loc).Length() > 4*0.003)
		{

			trial_start_time = now_time;
		}
	}
	if (inside_start_time == -1)
	{
		if (current_target == goal_target)
			inside_start_time = now_time;
	}

	
	if (rep_num > 0)
	{
		StampData(now_time);
	}

	
/*	new_spot = cursor_loc;
	Point vec = new_spot - old_spot;
	double dis = vec.Length();
	if (dis < 256*1.5*0.003)
		dis = 256*0.003;
	else if (dis < 256*2.5*0.003)
		dis = 256*2*0.003;
	else
		dis = 256*3*0.003;

	vec.Norm();
	new_spot = old_spot + dis*vec;

	double scale = 0.003;
	if (new_spot.x < 32*scale + 3*32*scale - BORDER_X || new_spot.x > -32*scale - 3*32*scale + BORDER_X
		|| new_spot.y < 32*scale + 3*32*scale - BORDER_Y || new_spot.y > -32*scale - 3*32*scale + BORDER_Y)
		new_spot.x = 100;

		
*/	
	
	DrawScene();











	CView::OnTimer(nIDEvent);

	// Eat spurious WM_TIMER messages
	MSG msg;
	while(::PeekMessage(&msg, m_hWnd, WM_TIMER, WM_TIMER, PM_REMOVE));
}

/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void CCubeView::Init()
{
	PIXELFORMATDESCRIPTOR pfd;
	int         n;
	HGLRC       hrc;
	GLfloat     fMaxObjSize;
	GLfloat     fNearPlane, fFarPlane;
	
	m_pDC = new CClientDC(this);

	ASSERT(m_pDC != NULL);

	if (!bSetupPixelFormat())
		return;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	CreateRGBPalette();
	freeze = 0;

	hrc = wglCreateContext(m_pDC->GetSafeHdc());
	wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

	GetClientRect(&m_oldRect);
	glClearDepth(1.0f);
	glDisable(GL_DEPTH_TEST);

	if (m_oldRect.bottom)
		fAspect = (GLfloat)m_oldRect.right/m_oldRect.bottom;
	else    // don't divide by zero, not that we should ever run into that...
		fAspect = 1.0f;
	fNearPlane = 3.0f;
	fFarPlane = 7.0f;
	fMaxObjSize = 3.0f;
	m_fRadius = fNearPlane + fMaxObjSize / 2.0f;

	CRect rect;
    GetClientRect(&rect);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.1,1.1,-1.1,1.1,-1.1,1.1);

	SetCursorPos(m_oldRect.CenterPoint().x,m_oldRect.CenterPoint().y);
	mouse_loc = Point(200,200);
	old_mouse_loc = Point(200,200);
	SetCursorPos(200,200);
	num_bubble_points = 0;

	SAFETY_ZONE = 0.05;

	bubble_frame = 0;
	fade_frame = 0;
	fade_target = -1;
	current_target = -1;
//	lock_cursor = 1;
	background =  WHITE_BACKGROUND;
	
	flatten_tim = 0;
	condition_num = 0;
	total_line_num = 0;

	user_set_targets = 30;
	last_stamp_time = 0;
	exp_line_num = 0;
	total_trials = 0;
	total_errors = 0;
	toggle_color = 0;
	do_another = 0;
	dontdraw = 0;

	sequence_num = 0;

	select_target = -1;
	drag_target = -1;


	cursor_loc = Point(0,0);
	cursor_theta1 = 0;
	cursor_theta2 = 2*M_PI;

	srand( (unsigned)time( NULL ) );


	MessageBox("       The Bubble Cursor Demo      \nWritten by Tovi Grossman  2005     \n       Dynamic Graphics Project     \n           University of Toronto     \n          www.dgp.toronto.edu     \n                -------------------     \n\nZ - Decrease targets     \nX - Increase targets     \nL - unlcok/lock windows cursor     \nQ - Square Targets (Beta!)     \n\nESCAPE - exit demo     \n", "Bubble Cursor");	

	SUBJECT_NUMBER = 12;

		

	if (SUBJECT_NUMBER == 1 || SUBJECT_NUMBER == 2)
	{
		cursor_order[0] = POINT;
		cursor_order[1] = TIM;
		cursor_order[2] = BUBBLE;
	}
	if (SUBJECT_NUMBER == 3 || SUBJECT_NUMBER == 4)
	{
		cursor_order[0] = POINT;
		cursor_order[1] = BUBBLE;
		cursor_order[2] = TIM;
	}
	if (SUBJECT_NUMBER == 5 || SUBJECT_NUMBER == 6)
	{
		cursor_order[0] = TIM;
		cursor_order[1] = POINT;
		cursor_order[2] = BUBBLE;
	}
	if (SUBJECT_NUMBER == 7 || SUBJECT_NUMBER == 8)
	{
		cursor_order[0] = TIM;
		cursor_order[1] = BUBBLE;
		cursor_order[2] = POINT;
	}
	if (SUBJECT_NUMBER == 9 || SUBJECT_NUMBER == 10)
	{
		cursor_order[0] = BUBBLE;
		cursor_order[1] = POINT;
		cursor_order[2] = TIM;
	}
	if (SUBJECT_NUMBER == 11 || SUBJECT_NUMBER == 12)
	{
		cursor_order[0] = BUBBLE;
		cursor_order[1] = TIM;
		cursor_order[2] = POINT;
	}



	cursor = cursor_order[do_another];
	do_another = 1;


	fitts_w[0] = 8;
	fitts_w[1] = 16;
	fitts_w[2] = 32;

	fitts_we[0] = (4.0/3.0);
	fitts_we[1] = 2;
	fitts_we[2] = 3;

	fitts_int[0] = 0;
	fitts_int[1] = 0.4;
	fitts_int[2] = 0.8;


	old_spot = Point(0,0);

	target_sequence[0][0] = Point(0.64206102, 0.42140437);
	target_sequence[0][1] = Point(-0.88662955, 0.57107447);
	target_sequence[0][2] = Point(1.09403985, -0.60594071);
	target_sequence[0][3] = Point(0.32806107, -0.66162291);
	target_sequence[0][4] = Point(1.14073265, 0.64178066);
	target_sequence[0][5] = Point(-1.16320785, 0.65833780);
	target_sequence[0][6] = Point(0.72382071, -0.66360744);
	target_sequence[0][7] = Point(1.16506643, -0.03501679);
	target_sequence[0][8] = Point(-0.34264538, -0.32844669);


	target_sequence[1][0] = Point(-0.40399526, 0.65315529);
	target_sequence[1][1] = Point(1.13200352, 0.65121949);
	target_sequence[1][2] = Point(-0.76941529, -0.64994249);
	target_sequence[1][3] = Point(0.03220105, 0.66028926);
	target_sequence[1][4] = Point(-0.73579576, 0.66250004);
	target_sequence[1][5] = Point(-1.16257704, 0.02400044);
	target_sequence[1][6] = Point(1.03611398, -0.66460235);
	target_sequence[1][7] = Point(-0.95036270, 0.50258504);
	target_sequence[1][8] = Point(0.57706049, 0.66467887);


	num_stamps = 0;
	lbutton = 0;

	warmup = 1;


	ShowCursor(0);


	double a = 0;
	double b = 0;
	double c = 0;
	for (int i = 0; i < 27; i++)
	{
		condition_list[i][0] = a;//int
		condition_list[i][1] = b;//w
		condition_list[i][2] = c;//we
		condition_list[i][3] = 0;//condition hasn't been done
		c++;
		if (c == 3)
		{
			c = 0;
			b++;
			if (b == 3)
			{
				b = 0;
				a++;
			}
		}
	}
	block_num = 0;


	num_conditions_complete = 0;
	NextCondition();
	//InitMultiTargets();
	InitTargets();
	goal_target = 0;
	CalcCurrentTarget();

	time_t rawtime;
	 struct tm * timeinfo;
	
	 char * weekday[] = { "Sunday", "Monday",
                       "Tuesday", "Wednesday",
                       "Thursday", "Friday", "Saturday"};
	char * month[] = { "Jan", "Feb","Mar", "Apr",  "May", "Jun", "Jul","Aug","Sep","Oct","Nov","Dec"};

 

	time ( &rawtime );
	timeinfo = localtime ( &rawtime );
 	sprintf(data_file, "Subject %d Session %d %s %s %d %d %d.%d.data",      SUBJECT_NUMBER, DAY_NUMBER, weekday[timeinfo->tm_wday],	month[timeinfo->tm_mon], timeinfo->tm_mday, timeinfo->tm_year+1900,timeinfo->tm_hour,timeinfo->tm_min);
	sprintf(point_file,"Subject %d Session %d %s %s %d %d %d.%d.allpoints", SUBJECT_NUMBER, DAY_NUMBER, weekday[timeinfo->tm_wday],	month[timeinfo->tm_mon], timeinfo->tm_mday, timeinfo->tm_year+1900,timeinfo->tm_hour,timeinfo->tm_min);	
	sprintf(total_file,"Subject %d Session %d %s %s %d %d %d.%d.totals",    SUBJECT_NUMBER, DAY_NUMBER, weekday[timeinfo->tm_wday],	month[timeinfo->tm_mon], timeinfo->tm_mday, timeinfo->tm_year+1900,timeinfo->tm_hour,timeinfo->tm_min);	
	
	


	SetTimer(1, 1, NULL);
	 
}

BOOL CCubeView::bSetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
		1,                              // version number
		PFD_DRAW_TO_WINDOW |            // support window
		  PFD_SUPPORT_OPENGL |          // support OpenGL
		  PFD_DOUBLEBUFFER,             // double buffered
		PFD_TYPE_RGBA,                  // RGBA type
		24,                             // 24-bit color depth
		0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // no alpha buffer
		0,                              // shift bit ignored
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // accum bits ignored
		32,                             // 32-bit z-buffer
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main layer
		0,                              // reserved
		0, 0, 0                         // layer masks ignored
	};
	int pixelformat;

	if ( (pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0 )
	{
		MessageBox("ChoosePixelFormat failed");
		return FALSE;
	}

	if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
	{
		MessageBox("SetPixelFormat failed");
		return FALSE;
	}

	return TRUE;
}

unsigned char CCubeView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
	unsigned char val;

	val = (unsigned char) (i >> shift);
	switch (nbits)
	{

	case 1:
		val &= 0x1;
		return oneto8[val];
	case 2:
		val &= 0x3;
		return twoto8[val];
	case 3:
		val &= 0x7;
		return threeto8[val];

	default:
		return 0;
	}
}


void CCubeView::CreateRGBPalette()
{
	PIXELFORMATDESCRIPTOR pfd;
	LOGPALETTE *pPal;
	int n, i;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	if (pfd.dwFlags & PFD_NEED_PALETTE)
	{
		n = 1 << pfd.cColorBits;
		pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

		ASSERT(pPal != NULL);

		pPal->palVersion = 0x300;
		pPal->palNumEntries = n;
		for (i=0; i<n; i++)
		{
			pPal->palPalEntry[i].peRed =
					ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
			pPal->palPalEntry[i].peGreen =
					ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
			pPal->palPalEntry[i].peBlue =
					ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
			pPal->palPalEntry[i].peFlags = 0;
		}

		/* fix up the palette to include the default GDI palette */
		if ((pfd.cColorBits == 8)                           &&
			(pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
			(pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
			(pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)
		   )
		{
			for (i = 1 ; i <= 12 ; i++)
				pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
		}

		m_cPalette.CreatePalette(pPal);
		delete pPal;

		m_pOldPalette = m_pDC->SelectPalette(&m_cPalette, FALSE);
		m_pDC->RealizePalette();
	}
}
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
void CCubeView::DrawScene(void)
{
	static BOOL     bBusy = FALSE;


	
    if(bBusy)
		return;
	if(dontdraw)
		return;

	bBusy = TRUE;

	glDrawBuffer (GL_BACK);
	if (oops)
	{
		glClearColor(0.5f, 0.0f, 0.0f, 1.0f);
		oops = 0;
	}
	else if (background == BLACK_BACKGROUND)
		glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	else
		glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

	glClearDepth(1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glDisable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
//	glViewport(0,0,m_oldRect.Width(), m_oldRect.Height());
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);



	glPushMatrix();

	glInitNames();
	glPushName(0);




///DRAW STUFF HERE///
	glLineWidth(1.0);


/*	glColor3f(1.0,1.0,1.0);
	glBegin(GL_POLYGON);
	glVertex3f(1,1,0);
	glVertex3f(-1,1,0);
	glVertex3f(-1,-1,0);
	glVertex3f(1,-1,0);
	glEnd();
*/
/*	glPushMatrix();
	glTranslatef(old_spot.x, old_spot.y,0);
	glutWireCube(0.01);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(new_spot.x, new_spot.y,0);
	glutWireCube(0.01);
	glPopMatrix();

	double scale = 0.003;
	glBegin(GL_LINE_STRIP);
	glVertex2f(32*scale + 3*32*scale - BORDER_X,32*scale + 3*32*scale - BORDER_Y);
	glVertex2f(32*scale + 3*32*scale - BORDER_X, -32*scale - 3*32*scale + BORDER_Y);
	glVertex2f(-32*scale - 3*32*scale + BORDER_X, -32*scale - 3*32*scale + BORDER_Y);
	glVertex2f(-32*scale - 3*32*scale + BORDER_X,32*scale + 3*32*scale - BORDER_Y);
	glVertex2f(32*scale + 3*32*scale - BORDER_X,32*scale + 3*32*scale - BORDER_Y);
	glEnd();
*/

	
	

	


	DrawTargets();
	DrawCursor();
/*	glBegin(GL_LINES);
	glVertex3f(test_point.x,test_point.y,0);
	glVertex3f((test_point+test_vec).x,(test_point+test_vec).y,0);
	glEnd();
*/



/*	glPushMatrix();
	glTranslatef(tim_loc.x,tim_loc.y,0);
	glBegin(GL_LINES);
	glVertex2f(0,0);
	glVertex2f(regress_vec.x,regress_vec.y);
	glEnd();
	glPopMatrix();*/


/*	glPushMatrix();
	glTranslatef(-tim_points[0].x,-tim_points[0].y,0);
	glPointSize(2.0);
	glBegin(GL_POINTS);
	for (int i = 0; i < NUM_TIM_POINTS; i++)
		glVertex2f(tim_points[i].x,tim_points[i].y);
	glEnd();
	glPopMatrix();*/


/////////////////////



	glPopMatrix();
	glFinish();	
	SwapBuffers(wglGetCurrentDC());	
	bBusy = FALSE;
		
}
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////


BOOL CCubeView::OnEraseBkgnd(CDC* pDC)
{
	return TRUE;
}


void CCubeView::InitLights()
{
	GLfloat light_position[] = { 0.0, 0.0, 1.0, 1.0}; 
	GLfloat spot_direction[] = {0.0, 0.0, -1.0};
	GLfloat light_color[] = {1.0,1.0,1.0,1.0};

	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spot_direction);
	//glLightfv(GL_LIGHT0, GL_AMBIENT, light_color);
	




	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glDepthFunc(GL_LEQUAL);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	
}







void CCubeView::OnUpdateAppCalibrate(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}



static int mousehasmoved = 0;


///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//		          Cursor Calling Functions               //
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void CCubeView::OnMouseMove(UINT nFlags, CPoint point) 
{








	CView::OnMouseMove(nFlags, point);
}

void CCubeView::OnLButtonDown(UINT nFlags, CPoint point) 
{

	if (freeze)
		return;

	double t = GetSeconds();
	lbutton = 1;
	if (rep_num > 0)
		StampData(t);
	if (current_target == goal_target)
		TargetHit(t);
	else
		TargetMiss(t);



	lbutton = 1;

//	TRACE("%2.8f, %2.8f\n", new_spot.x, new_spot.y);
//	old_spot = new_spot;

	
	
	CView::OnLButtonDown(nFlags, point);
}
void CCubeView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	lbutton = 0;






	CView::OnLButtonUp(nFlags, point);
}

void CCubeView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	rbutton = 1;
/*	drag_target = -1;
	select_target = -1;
	size_target = -1;
	if (current_target >= 0 && !currently_creating)
	{
		size_target = current_target;
		size_start = cursor_loc;
		size_target_start_x = target_dims[current_target][0];
		size_target_start_y = target_dims[current_target][1];
	}*/

	CView::OnRButtonDown(nFlags, point);	
}



void CCubeView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	rbutton = 0;


	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonUp(nFlags, point);
}


void CCubeView::DrawCursor()
{
	if (cursor == BUBBLE)
		DrawBubbleCursor();
	else if (cursor == AREA)
		DrawAreaCursor();
	else if (cursor == POINT)
		DrawPointCursor();



}
void CCubeView::DrawPointCursor()
{
	glLineWidth(1.0);
	if (background == WHITE_BACKGROUND)
		glColor4f(0.35,0.35,0.35,1);
	else
		glColor4f(1,1,1,1);
	double s = 0.03;
	glPushMatrix();
	glTranslatef(cursor_loc.x,cursor_loc.y,.9);
	glBegin(GL_LINES);
	glVertex3f(-1*s/2,0,.1);
	glVertex3f(s/2,0,.1);
	glVertex3f(0,-1*s/2,.1);
	glVertex3f(0,s/2,.1);
	glEnd();
	glPopMatrix();

/*	if (current_target != -1)
	{
		glColor4f(0.1,0.1,1.0,0.65);
		glPushMatrix();
		glTranslatef(target_locs[current_target].x,target_locs[current_target].y,.9);

		if (target_dims[current_target][2] == CIRCLE)
		{
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			double r = target_dims[current_target][0] + 0.001;
			double a = 0;
			double seg_length = 2*M_PI;
			int loop = 1;
			double tot_rad = 0;
			while(1)
			{		
				double x = r*sin(a);
				double y = r*cos(a);
				glVertex3f(x,y,0.1);
				if (loop == 0)
					break;
				a += 0.04;
				tot_rad += 0.04;
				if (tot_rad >= seg_length)
				{
					a = 2*M_PI;
					loop = 0;
				}
			}
			glEnd();
			glPopMatrix();
		}
		else if (target_dims[current_target][2] == SQUARE)
		{
			double x = target_dims[current_target][0]/2;
			double y = target_dims[current_target][1]/2;

			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			glVertex3f(x,y,0.1);
			glVertex3f(x,-y,0.1);
			glVertex3f(-x,-y,0.1);
			glVertex3f(-x,y,0.1);
			glEnd();
			glPopMatrix();
		}
	}*/


}

void CCubeView::DrawAreaCursor()
{
	glLineWidth(1.0);
	glColor4f(1,1,1,1);

	double s = 0.2;
	glPushMatrix();
	glTranslatef(cursor_loc.x,cursor_loc.y,.9);
	glBegin(GL_LINES);
	glVertex3f(-1*s/2,0,.1);
	glVertex3f(s/2,0,.1);
	glVertex3f(0,-1*s/2,.1);
	glVertex3f(0,s/2,.1);
	glEnd();


/*	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
	glBegin(GL_POLYGON);
	glVertex3f(-0.1,-0.1,0.1);
	glVertex3f(-0.1,0.1,0.1);
	glVertex3f(0.1,0.1,0.1);
	glVertex3f(0.1,-0.1,0.1);
	glEnd();
	glPopMatrix();*/


		
		glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
		glBegin(GL_POLYGON);
		double r = cursor_rad;
		double a = 0;
		double seg_length = 2*M_PI;
		int loop = 1;
		double tot_rad = 0;
		while(1)
		{		
			double x = r*sin(a);
			double y = r*cos(a);
			glVertex3f(x,y,0.1);
			if (loop == 0)
				break;
			a += 0.04;
			tot_rad += 0.04;
			if (tot_rad >= seg_length)
			{
				a = 2*M_PI;
				loop = 0;
			}
		}
		glEnd();
		glPopMatrix();



	if (current_target != -1)
	{
		glColor4f(0.1,0.1,1.0,0.65);
		glPushMatrix();
		glTranslatef(target_locs[current_target].x,target_locs[current_target].y,.9);

	
		if (target_dims[current_target][2] == CIRCLE)
		{
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			double r = target_dims[current_target][0] + 0.001;
			double a = 0;
			double seg_length = 2*M_PI;
			int loop = 1;
			double tot_rad = 0;
			while(1)
			{		
				double x = r*sin(a);
				double y = r*cos(a);
				glVertex3f(x,y,0.1);
				if (loop == 0)
					break;
				a += 0.04;
				tot_rad += 0.04;
				if (tot_rad >= seg_length)
				{
					a = 2*M_PI;
					loop = 0;
				}
			}
			glEnd();
			glPopMatrix();
		}
		else if (target_dims[current_target][2] == SQUARE)
		{
			double x = target_dims[current_target][0]/2;
			double y = target_dims[current_target][1]/2;

			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			glVertex3f(x,y,0.1);
			glVertex3f(x,-y,0.1);
			glVertex3f(-x,-y,0.1);
			glVertex3f(-x,y,0.1);
			glEnd();
			glPopMatrix();
		}
	}




}

void CCubeView::DrawBubbleCursor()
{



	double r = cursor_rad;
//	double s = 0.15 - r/2;
//	if (s < 0.03)
	double s = 0.08;
	glLineWidth(2.5);
	glColor4f(1,1,1,1);
	if (background == WHITE_BACKGROUND)
		glColor4f(0.0,0.0,0.0,1);
	else
		glColor4f(1,1,1,1);

	glPushMatrix();
	glTranslatef(cursor_loc.x,cursor_loc.y,.9);
	glBegin(GL_LINES);
	glVertex3f(-1*s/2,0,.1);
	glVertex3f(s/2,0,.1);
	glVertex3f(0,-1*s/2,.1);
	glVertex3f(0,s/2,.1);
	glEnd();



	if (background == BLACK_BACKGROUND)
		glColor4f(0.0,0.0,1.0,0.4);
	else
		glColor4f(0.0,0.0,0.0,0.4);


	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glBegin(GL_POLYGON);
	double a = cursor_theta1;
	double seg_length = cursor_theta2 - cursor_theta1;
	if (seg_length < 0)
		seg_length += 2*M_PI;
	int loop = 1;
	double tot_rad = 0;
	while(1)
	{		
		double x = r*sin(a);
		double y = r*cos(a);
		glVertex3f(x,y,0.1);
		if (loop == 0)
			break;
		a += 0.04;
		tot_rad += 0.04;
		if (a > 2*M_PI)
			a -= 2*M_PI;
		if (tot_rad >= seg_length)
		{
			a = cursor_theta2;
			loop = 0;
		}
	}
	glEnd();




	glPopMatrix();

	
	
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	glBegin(GL_POLYGON);
	for (int i = 0; i < num_bubble_points; i++)
	{
		bubble_points[i] = start_bubble_points[i] + ((double)bubble_frame/(double)NUM_BUBBLE_FRAMES)*(final_bubble_points[i] - start_bubble_points[i]);
		glVertex3f(bubble_points[i].x,bubble_points[i].y,0.1);
	}
	glEnd();






/*	if (target_dims[current_target][2] == SQUARE)
	{
		Point q = target_locs[current_target];
		Point v = q - cursor_loc;
		if (fabs(v.x) < target_dims[current_target][0]/2.0  && fabs(v.y) < target_dims[current_target][1]/2.0)
		{	
			double x = target_dims[current_target][0]/2;
			double y = target_dims[current_target][1]/2;
			glPushMatrix();
			glTranslatef(target_locs[current_target].x,target_locs[current_target].y,.9);


			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			glVertex3f(x,y,0.1);
			glVertex3f(x,-y,0.1);
			glVertex3f(-x,-y,0.1);
			glVertex3f(-x,y,0.1);
			glEnd();
			glPopMatrix();
		}
	}*/

	if (current_target != -1)
	{
		if (background == BLACK_BACKGROUND)
			glColor4f(0.0,0.0,1.0,0.4);
		else
			glColor4f(0.0,0.0,0.0,0.4);
		glPushMatrix();
		glTranslatef(target_locs[current_target].x,target_locs[current_target].y,.9);

		if (target_dims[current_target][2] == CIRCLE && (cursor_loc-target_locs[current_target]).Length() < target_dims[current_target][0])
		{

			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			double r = target_dims[current_target][0] + 0.001;
			double a = 0;
			double seg_length = 2*M_PI;
			int loop = 1;
			double tot_rad = 0;
			while(1)
			{		
				double x = r*sin(a);
				double y = r*cos(a);
				glVertex3f(x,y,0.1);
				if (loop == 0)
					break;
				a += 0.04;
				tot_rad += 0.04;
				if (tot_rad >= seg_length)
				{
					a = 2*M_PI;
					loop = 0;
				}
			}
			glEnd();
		}
	/*	else if (target_dims[current_target][2] == SQUARE && (cursor_loc-target_locs[current_target]).Length() < target_dims[current_target][0])
		{
			double x = target_dims[current_target][0]/2;
			double y = target_dims[current_target][1]/2;

			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
			glBegin(GL_POLYGON);
			glVertex3f(x,y,0.1);
			glVertex3f(x,-y,0.1);
			glVertex3f(-x,-y,0.1);
			glVertex3f(-x,y,0.1);
			glEnd();

		}*/
		glPopMatrix();
	}





	
	




	


}
void CCubeView::CalcCurrentTarget()
{
	if (cursor == BUBBLE)
		CalcCursorRad();
	else if (cursor == TIM)
		CalcTimTarget();
	else
	{
		current_target = -1;
		if (cursor == AREA)
			CalcAreaTarget();
		if (current_target == -1)
			CalcPointTarget();
	}
}
void CCubeView::CalcPointTarget()
{
	Point p = cursor_loc;
	for (int i = 0; i < num_targets; i++)
	{
		if (target_dims[i][2] == CIRCLE)
		{
			Point q = target_locs[i];
			double d= (p-q).Length();
			if (d < target_dims[i][0])
			{
				current_target = i;
				return;
			}
		}
		else if (target_dims[i][2] == SQUARE)
		{
			Point q = target_locs[i];
			Point v = q - p;
			if (fabs(v.x) < target_dims[i][0]/2.0  && fabs(v.y) < target_dims[i][1]/2.0)
			{
				current_target = i;
				return;
			}
		}
	}

}
void CCubeView::CalcAreaTarget()
{
	int temp = -1;
	Point p = cursor_loc;
	for (int i = 0; i < num_targets; i++)
	{
		if (target_dims[i][2] == CIRCLE)
		{

			Point q = target_locs[i];
			double d= (p-q).Length();
			d = d - cursor_rad;
			if (d < target_dims[i][0])
			{
				if (temp == -1)
					temp = i;
				else
					return;
			}
		}
		else if (target_dims[i][2] == SQUARE)
		{
			Point q = target_locs[i];
			Point v = q - p;
			Point d;
			if (fabs(v.x) < target_dims[i][0]/2.0)
				d.x = 0;
			else
				d.x = fabs(v.x) - target_dims[i][0]/2.0;
			if (fabs(v.y) < target_dims[i][1]/2.0)
				d.y = 0;
			else
				d.y = fabs(v.y) - target_dims[i][1]/2.0;


			if (d.Length() < cursor_rad )
			{
				if (temp == -1)
					temp = i;
				else
					return;
			}
		}
	}
	current_target = temp;



}

void CCubeView::CalcCursorRad()
{
	

	num_bubble_points = 0;
	///closest edge
	double closest_edge_dist = 1000;
	int closest_target = -1;
	Point p = cursor_loc;

	for (int i = 0; i < num_targets; i++)
	{
		if (target_dims[i][2] == CIRCLE)
		{
			Point q = target_locs[i];
			double d= (p-q).Length();
			d = d - target_dims[i][0];
			if (d < closest_edge_dist)
			{
				closest_edge_dist = d;
				closest_target = i;
			}
		}
		else if (target_dims[i][2] == SQUARE)
		{
			double rev = 1;
			Point q = target_locs[i];
			Point v = q - p;
			Point d;
			if (fabs(v.x) < target_dims[i][0]/2.0)
				d.x = 0;
			else
				d.x = fabs(v.x) - target_dims[i][0]/2.0;
			if (fabs(v.y) < target_dims[i][1]/2.0)
				d.y = 0;
			else
				d.y = fabs(v.y) - target_dims[i][1]/2.0;

			if (d.x == 0 && d.y == 0)
			{
				d.x = fabs(v.x) - target_dims[i][0]/2.0;
				d.y = fabs(v.y) - target_dims[i][1]/2.0;
				if (d.x < d.y)
					d.y = 0;
				else
					d.x = 0;
				rev = -1;
			}
			double len = d.Length()*rev;




			if (len < closest_edge_dist )
			{
				closest_edge_dist = len;
				closest_target = i;
			}
		}
	}

	if (current_target != closest_target)
	{
		fade_target = current_target;
		fade_frame = 0;
		bubble_frame = 0;
	}
	current_target = closest_target;
	double final_rad;

	if (target_dims[closest_target][2] == CIRCLE)
		final_rad = closest_edge_dist + 2*target_dims[closest_target][0];
	else if (target_dims[closest_target][2] == SQUARE)
	{
		Point q = target_locs[closest_target];
		Point v = q - p;
		v.x = fabs(v.x);
		v.y = fabs(v.y);
		Point corner = Point(-1*target_dims[closest_target][0]/2,-1*target_dims[closest_target][1]/2);
		final_rad = (corner - v).Length();
	}





	make_bubble = false;



	for (int i = 0; i < num_targets; i++)
	{
		if (i != closest_target)
		{
			double d;
			if (target_dims[i][2] == CIRCLE)
			{
				Point q = target_locs[i];
				d = (p-q).Length();
				d = d - target_dims[i][0];
				d = d - (target_dims[i][0]/2);
				if (d < 0)
					d += (target_dims[i][0]/2);
			//	if (d < closest_edge_dist + 0.0001)
			//		d = closest_edge_dist + 0.0001;


			}
			else if (target_dims[i][2] == SQUARE)
			{
				double rev = 1;
				Point q = target_locs[i];
				Point v = q - p;
				Point delta;
				if (fabs(v.x) < target_dims[i][0]/2.0)
					delta.x = 0;
				else
					delta.x = fabs(v.x) - target_dims[i][0]/2.0;
				if (fabs(v.y) < target_dims[i][1]/2.0)
					delta.y = 0;
				else
					delta.y = fabs(v.y) - target_dims[i][1]/2.0;


				d = delta.Length();
				d = d - 0.05;//((target_dims[i][0]+target_dims[i][1])/4);
				if (d < 0)
					d = 0;//	d = d + ((target_dims[i][0]+target_dims[i][1])/4);
			}



			if (d < final_rad)
			{
				final_rad = d;
				make_bubble = true;
			}
		}
	}

	cursor_rad = final_rad;
	if (closest_edge_dist >= 0)
	{
		if ( cursor_rad < closest_edge_dist+0.0001)			
			cursor_rad = closest_edge_dist+0.0001;
	}
	else if ( cursor_rad < fabs(closest_edge_dist+0.0001))
		cursor_rad = fabs(closest_edge_dist)+0.0001;



	if (!make_bubble || closest_edge_dist < 0)
	{
		bubble_frame = NUM_BUBBLE_FRAMES;
		cursor_theta1 = 0;
		cursor_theta2 = 2*M_PI;
		return;
	}



	Point i1, i2;
	int num_int_points;
	Point int_points[8];
	if (target_dims[closest_target][2] == CIRCLE)
		GetCircleCircleIntersection(cursor_loc,cursor_rad,target_locs[closest_target],target_dims[closest_target][0], &i1, &i2);
	else if (target_dims[closest_target][2] == SQUARE)
		num_int_points = GetCircleSquareIntersection(cursor_loc, cursor_rad, target_locs[closest_target], target_dims[closest_target][0], target_dims[closest_target][1], int_points);

	if (target_dims[closest_target][2] == CIRCLE)
	{
		double theta_a1,theta_a2;
		double theta1;
		theta_a1 = asin((i1.x - cursor_loc.x)/cursor_rad);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;


		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		double theta_b1,theta_b2;
		theta_b1 = acos((i1.y - cursor_loc.y)/cursor_rad);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta1 = theta_a1;
		else
			theta1 = theta_a2;


	//	Point t1;
	//	t1.x = cursor_loc.x+cursor_rad*sin(theta1);
	//	t1.y = cursor_loc.y+cursor_rad*cos(theta1);



		double theta2;
		theta_a1 = asin((i2.x - cursor_loc.x)/cursor_rad);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;


		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		theta_b1,theta_b2;
		theta_b1 = acos((i2.y - cursor_loc.y)/cursor_rad);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta2 = theta_a1;
		else
			theta2 = theta_a2;

	//	Point t2;
	//	t2.x = cursor_loc.x+cursor_rad*sin(theta2);
	//	t2.y = cursor_loc.y+cursor_rad*cos(theta2);


		Point i3 = target_locs[closest_target];
		double temp_d = (cursor_loc-target_locs[closest_target]).Length();
		

		double theta3;
		theta_a1 = asin((i3.x - cursor_loc.x)/temp_d);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;


		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		theta_b1,theta_b2;
		theta_b1 = acos((i3.y - cursor_loc.y)/temp_d);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta3 = theta_a1;
		else
			theta3 = theta_a2;

	//	Point t3;
	//	t3.x = cursor_loc.x+temp_d*sin(theta3);
	//	t3.y = cursor_loc.y+temp_d*cos(theta3);

		if (AngleInside(theta3,theta1,theta2))
		{
			double temp = theta1;
			theta1 = theta2;
			theta2 = temp;
		}
		cursor_theta1 = theta1;
		cursor_theta2 = theta2;
	//	cursor_theta1 += tan(0.01/(cursor_rad));
		cursor_theta1 += tan((target_dims[current_target][0]/2.4)/(cursor_rad));
		cursor_theta2 -= tan((target_dims[current_target][0]/2.4)/(cursor_rad));
	


	
	
		//make the bubble

		Point q1;
		q1.x = cursor_loc.x+cursor_rad*sin(cursor_theta1);			
		q1.y = cursor_loc.y+cursor_rad*cos(cursor_theta1);
		Point q2;
		q2.x = cursor_loc.x+cursor_rad*sin(cursor_theta2);			
		q2.y = cursor_loc.y+cursor_rad*cos(cursor_theta2);
		bubble_rad = (q1-target_locs[closest_target]).Length();

		Point j1, j2, j3;
		double seg_length = GetCiricleSegmentLength(target_locs[closest_target],bubble_rad,q1, q2, cursor_loc);
		double spread = 0.01;
		num_bubble_points = seg_length/spread;

		i1 = q1;
		i2 = q2;



		theta_a1 = asin((i1.x - target_locs[closest_target].x)/bubble_rad);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;

		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		theta_b1,theta_b2;
		theta_b1 = acos((i1.y - target_locs[closest_target].y)/bubble_rad);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta1 = theta_a1;
		else
			theta1 = theta_a2;


		//t1.x = target_locs[closest_target].x+bubble_rad*sin(theta1);
		//t1.y = target_locs[closest_target].y+bubble_rad*cos(theta1);



		theta_a1 = asin((i2.x - target_locs[closest_target].x)/bubble_rad);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;


		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		theta_b1,theta_b2;
		theta_b1 = acos((i2.y - target_locs[closest_target].y)/bubble_rad);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta2 = theta_a1;
		else
			theta2 = theta_a2;


	//	t2.x = target_locs[closest_target].x+bubble_rad*sin(theta2);
	//	t2.y = target_locs[closest_target].y+bubble_rad*cos(theta2);

		
		i3 = cursor_loc;
		temp_d = (cursor_loc-target_locs[closest_target]).Length();
		

		theta_a1 = asin((i3.x - target_locs[closest_target].x)/temp_d);
		while(theta_a1 < 0)
			theta_a1 += 2*M_PI;


		if (theta_a1 <= M_PI)
			theta_a2 = M_PI - theta_a1;
		else
			theta_a2 = 3*M_PI - theta_a1;

		while(theta_a2 < 0)
			theta_a2 += 2*M_PI;

		theta_b1,theta_b2;
		theta_b1 = acos((i3.y - target_locs[closest_target].y)/temp_d);
		while(theta_b1 < 0)
			theta_b1 += 2*M_PI;
		theta_b2 = 2*M_PI - theta_b1;
		while(theta_b2 < 0)
			theta_b2 += 2*M_PI;

		if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
			theta3 = theta_a1;
		else
			theta3 = theta_a2;


	//	t3.x = target_locs[closest_target].x+temp_d*sin(theta3);
	//	t3.y = target_locs[closest_target].y+temp_d*cos(theta3);

		if (AngleInside(theta3,theta1,theta2))
		{
			double temp = theta1;
			theta1 = theta2;
			theta2 = temp;
		}
		bubble_theta1 = theta1;
		bubble_theta2 = theta2;

		CreateBubblePoints(closest_target);
	}
	else if (target_dims[closest_target][2] == SQUARE)
	{

		if (num_int_points == 1)
		{
			int_points[1] = int_points[0];
			num_int_points = 2;
		}
		double int_point_thetas[8];
		double theta1, theta2;
		theta1 = 0;
		theta2 = 2*M_PI;
		for (int i = 0; i < num_int_points; i++)
			int_point_thetas[i] = GetTheta(cursor_loc, cursor_rad, int_points[i]);
		double midang;
		Point midp;
		midp = int_points[0] + 0.5*(int_points[1] - int_points[0]);	
		midang = GetTheta(cursor_loc, (midp-cursor_loc).Length(), midp);
		for (int i = 0; i < num_int_points; i++)
		{
			for (int j = 0; j < num_int_points; j++)
			{
				if (i != j)
				{
					bool accept = true;
					for (int k = 0; k < num_int_points; k++)
					{
						if (k != i && k != j)
						{
							if (!AngleInside(int_point_thetas[k], int_point_thetas[i], int_point_thetas[j]))
								accept = false;
						}

					}
					if (accept && !AngleInside(midang, int_point_thetas[i], int_point_thetas[j]))
						accept = false;
					if (accept == true)
					{
						theta1 = int_point_thetas[j];
						theta2 = int_point_thetas[i];
					}
				}

			}
		}
//		TRACE("%2.2f, %2.2f\n", theta1, theta2);
		cursor_theta1 = theta1;
		cursor_theta2 = theta2;

	//	cursor_theta1 += tan((target_dims[current_target][0]/4.8)/(cursor_rad));
	//	cursor_theta2 -= tan((target_dims[current_target][0]/4.8)/(cursor_rad));
	




		//make the square bubble

		Point q1;
		q1.x = cursor_loc.x+cursor_rad*sin(cursor_theta1);			
		q1.y = cursor_loc.y+cursor_rad*cos(cursor_theta1);
		Point q2;
		q2.x = cursor_loc.x+cursor_rad*sin(cursor_theta2);			
		q2.y = cursor_loc.y+cursor_rad*cos(cursor_theta2);

		bubble_w = target_dims[closest_target][0];
		double temp_w = fabs(q1.x - target_locs[closest_target].x)*2;
		if (temp_w > bubble_w)
			bubble_w = temp_w;
		temp_w = fabs(q2.x - target_locs[closest_target].x)*2;
		if (temp_w > bubble_w)
			bubble_w = temp_w;

		bubble_h = target_dims[closest_target][1];
		double temp_h = fabs(q1.y - target_locs[closest_target].y)*2;
		if (temp_h > bubble_h)
			bubble_h = temp_h;
		temp_h = fabs(q2.y - target_locs[closest_target].y)*2;
		if (temp_h > bubble_h)
			bubble_h = temp_h;



		Point j1, j2, j3;
		double seg_length = GetSquareSegmentLength(target_locs[closest_target],bubble_w, bubble_h,q1, q2, cursor_loc);
		if (seg_length < 0.001)
		{
			make_bubble = false;
			bubble_frame = NUM_BUBBLE_FRAMES;
			cursor_theta1 = 0;
			cursor_theta2 = 2*M_PI;
			return;
		}
		double spread = 0.01;
		bubble_seg_length = seg_length;
		num_bubble_points = seg_length/spread;

		i1 = q1;
		i2 = q2;


		double temp_d = (i1-target_locs[closest_target]).Length();
		theta1 = GetTheta(target_locs[closest_target],temp_d,i1);
		temp_d = (i2-target_locs[closest_target]).Length();
		theta2 = GetTheta(target_locs[closest_target],temp_d,i2);
		
		Point i3 = i1 + 0.5*(i2 - i1);
		
		

		temp_d = (i3-target_locs[closest_target]).Length();
		double theta3 = GetTheta(target_locs[closest_target],temp_d,i3);
		

		if (AngleInside(theta3,theta1,theta2))
		{
			Point temp = i1;
			i1 = i2;
			i2 = temp;
		}


		CreateBubblePoints(closest_target);


	}



}
double CCubeView::GetTheta(Point p1, double r, Point i1)
{
	double theta_a1,theta_a2;
	double theta1;
	double k = (i1.x - p1.x)/r;
	if (k > 1)
		k = 1;
	if (k < -1)
		k = -1;

	theta_a1 = asin(k);
	while(theta_a1 < 0)
		theta_a1 += 2*M_PI;


	if (theta_a1 <= M_PI)
		theta_a2 = M_PI - theta_a1;
	else
		theta_a2 = 3*M_PI - theta_a1;

	while(theta_a2 < 0)
		theta_a2 += 2*M_PI;

	double theta_b1,theta_b2;
	k = (i1.y - p1.y)/r;
	if (k > 1)
		k = 1;
	if (k < -1)
		k = -1;

	theta_b1 = acos(k);
	while(theta_b1 < 0)
		theta_b1 += 2*M_PI;
	theta_b2 = 2*M_PI - theta_b1;
	while(theta_b2 < 0)
		theta_b2 += 2*M_PI;

	if (fabs(theta_a1 - theta_b1) < 0.000001 || fabs(theta_a1 - theta_b2) < 0.000001)
		theta1 = theta_a1;
	else
		theta1 = theta_a2;
	return theta1;







}

void CCubeView::CreateBubblePoints(int closest_target)
{
	if (target_dims[closest_target][2] == CIRCLE)
	{
		double a = bubble_theta1;
		Point c = target_locs[closest_target];
		double r = bubble_rad;
		int i = 0;
		double total_theta = 0;
		double seg_length = bubble_theta2 - bubble_theta1;
		if (seg_length < 0)
			seg_length += 2*M_PI;
		double inc = (seg_length/(num_bubble_points));
		bool loop = 1;
		while (1)
		{
			Point bp;
			bp.x = c.x + r*sin(a);
			bp.y = c.y + r*cos(a);
			final_bubble_points[i] = bp;
			i++;
			if (loop == 0)
				break;
			a += inc;
			total_theta += inc;
			if (a > 2*M_PI)
				a -= 2*M_PI;
			if (total_theta >= seg_length)
			{
				a = bubble_theta2;
				loop = 0;
			}
		}
		num_bubble_points = i;


		a = cursor_theta2;
		c = cursor_loc;
		r = cursor_rad;
		total_theta = 0;
		seg_length = cursor_theta1 - cursor_theta2;
		if (seg_length < 0)
			seg_length += 2*M_PI;
		inc = (seg_length/(num_bubble_points));
		loop = 1;
		i = 0;
		while (1)
		{
			Point bp;
			bp.x = c.x + r*sin(a);
			bp.y = c.y + r*cos(a);
			start_bubble_points[i] = bp;
			i++;
			if (loop == 0)
				break;
			a += inc;
			total_theta += inc;
			if (a > 2*M_PI)
				a -= 2*M_PI;
			if (i == num_bubble_points-1)
			{
				a = cursor_theta1;
				loop = 0;
			}
		}
		if (i != num_bubble_points)
			TRACE("MISMATCH!\n");
	}
	if (target_dims[closest_target][2] == SQUARE)
	{
		int i = 0;
		double seg_length = bubble_seg_length;
		double inc = (seg_length/(num_bubble_points));
		Point p = bubble_point1;
		bool loop = 1;
		double total_length = 0;
		while (1)
		{
			Point bp;
			final_bubble_points[i] = p;
			i++;
			if (loop == 0)
				break;
			p = NextSquarePoint(p,  inc, target_locs[closest_target], bubble_w, bubble_h);
			total_length += inc;
			if (total_length >= seg_length)
			{
				p = bubble_point2;
				loop = 0;
			}
		}
		num_bubble_points = i;

		double a = cursor_theta2;
		Point c = cursor_loc;
		double r = cursor_rad;
		double total_theta = 0;
		seg_length = cursor_theta1 - cursor_theta2;
		if (seg_length < 0)
			seg_length += 2*M_PI;
		inc = (seg_length/(num_bubble_points));
		loop = 1;
		i = 0;
		while (1)
		{
			Point bp;
			bp.x = c.x + r*sin(a);
			bp.y = c.y + r*cos(a);
			start_bubble_points[i] = bp;
			i++;
			if (loop == 0)
				break;
			a += inc;
			total_theta += inc;
			if (a > 2*M_PI)
				a -= 2*M_PI;
			if (i == num_bubble_points-1)
			{
				a = cursor_theta1;
				loop = 0;
			}
		}
		if (i != num_bubble_points)
			TRACE("MISMATCH!\n");

		





	}

}

Point CCubeView::NextSquarePoint(Point p,  double inc, Point p0, double  w, double h)
{
	Point q = NextCorner(p, p0, w, h);
	double d = (q-p).Length();
	if (d < inc)
	{
		p = q;
		inc -= d;
	}

	Point c = p0;
	double minx = c.x - (w/2);
	double maxx = c.x + (w/2);

	double miny = c.y - (h/2);
	double maxy = c.y + (h/2);

	if (fabs(p.x - maxx) < 0.0001 && p.y > miny)
		p.y -= inc;
	else if (fabs(p.y - miny) < 0.0001 && p.x > minx)
		p.x -= inc;
	else if (fabs(p.x - minx) < 0.0001 && p.y < maxy)
		p.y += inc;
	else 
		p.x += inc;

	return p;

}



void CCubeView::InitTargets()
{
	num_targets = 0;
	Point new_target_loc;
	double new_target_rad;


	while (num_targets < user_set_targets)
	{
		double x, y, r;
		int d1 = rand();
		x = (double)d1;
		x = x/32767.0;
		x = x*(3.0) - 1.5;
		d1 = rand();
		y = (double)d1;
		y = y/32767.0;
		y = y*(1.8) - 0.9;
		d1 = rand();
		r = (double)d1;
		r = r/32767.0;
		r = r*(0.1) + 0.02;

		new_target_loc = Point(x,y);
		new_target_rad = r;

		bool accept = true;
		for (int i = 0; i < num_targets; i++)
		{
			if (target_dims[i][2] == CIRCLE)
			{
				double len = (new_target_loc - target_locs[i]).Length() - new_target_rad - target_dims[i][0];
				if (len <= 0.08)
				{
					accept = false;
					i = 9099;
				}
			}
			else if (target_dims[i][2] == SQUARE)
			{
				double len = (new_target_loc - target_locs[i]).Length() - new_target_rad - target_dims[i][0];
				if (len <= 0.08)
				{
					accept = false;
					i = 9099;
				}
			}
		}
		if (accept)
		{
			target_locs[num_targets] = new_target_loc;
			target_dims[num_targets][0] = new_target_rad;
			target_dims[num_targets][1] = new_target_rad;
			target_dims[num_targets][2] = CIRCLE;

			num_targets++;
		}
	
	}

	num_selectable_targets = TOTAL_TARGETS;







/*

	target_locs[num_targets] = Point(-0.5,-0.4);
	num_targets++;
	target_locs[num_targets] = Point(-0.1,0.4);
	num_targets++;
	target_locs[num_targets] = Point(0.5,-0.8);
	num_targets++;
	target_locs[num_targets] = Point(-0.2,-0.7);
	num_targets++;
	target_locs[num_targets] = Point(0.2,-0.1);
	num_targets++;
	target_locs[num_targets] = Point(-0.5,0.6);
	num_targets++;


	target_locs[num_targets] = Point(-0.7,-0.2);
	num_targets++;
	target_locs[num_targets] = Point(-0.7,0.3);
	num_targets++;
	target_locs[num_targets] = Point(0.5,0.3);
	num_targets++;
	target_locs[num_targets] = Point(0.2,-0.4);
	num_targets++;
	target_locs[num_targets] = Point(0.6,-0.1);
	num_targets++;
	target_locs[num_targets] = Point(-0.1,0.7);
	num_targets++;


	for (int i = 0; i < num_targets; i++)
		target_rads[i] = ((double)i + 1)/80;*/
	flatten_tim = 0;
}


void CCubeView::InitFittsTargets()
{


	num_targets = 0;

	double goal1;
	double goal2;
	double d=0;
	double w;
	double we;

	bool center_target;
	///16 128 1024



	w = fitts_w[condition_list[condition_num][1]];
	we = fitts_we[condition_list[condition_num][2]];
//	d = fitts_a[condition_list[condition_num][0]];


	double r1 = w/2;
	double r2;
	double obstacle1;
	double obstacle2;

	goal1 = d/2;
	goal2 = -d/2;

	if (d/2 > we + w/2)
	{
		center_target = 0;
		r2 = r1;
		obstacle1 = goal1 - we;
		obstacle2 = -1*obstacle1;
	}
	else
	{
		center_target = 1;
		r2 = goal1 - we + r1;
		obstacle1 = 0;
	}





	target_locs[num_targets] = Point(goal1,0);
	target_dims[num_targets][0] = r1;
	target_dims[num_targets][1] = r1;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;
	target_locs[num_targets] = Point(goal2,0);
	target_dims[num_targets][0] = r1;
	target_dims[num_targets][1] = r1;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;

	
/*
	target_locs[num_targets] = Point(obstacle1,0);
	target_rads[num_targets] = r2;
	num_targets++;


	target_locs[num_targets] = Point(goal1 + goal1 - obstacle1,0);
	target_rads[num_targets] = r2;
	num_targets++;


	target_locs[num_targets] = Point(-1*(goal1 + goal1 - obstacle1),0);
	target_rads[num_targets] = r2;
	num_targets++;





	if (!center_target)
	{
		target_locs[num_targets] = Point(obstacle2,0);
		target_rads[num_targets] = r2;
		num_targets++;
	}

	double check = (goal1 + obstacle1 - r1 - r2)/2;
	TRACE("%2.2f\n",(goal1 - r1 - check)*2);
*/
	double scale = 0.003;
	for (int i = 0; i < num_targets; i++)
	{
		target_locs[i] = scale * target_locs[i];
		target_dims[i][0] = scale * target_dims[i][0];
		target_dims[i][1] = scale * target_dims[i][0];
	}


	num_selectable_targets = 2;

	Point p, q, p2, q2;
	p.x = scale*(goal1 - we/2.0);
	p.y = 0;
	double wr = scale*(we/2.0 - r1);
	q = -1*p;


	p2.x = scale*(goal1 + we/2.0);
	p2.y = 0;
	q2 = -1*p2;

	Point vec;

	int counter = 1;
	while (counter <= 4)
	{
		
		Point temppoint;
		if (counter == 1)
			temppoint = p;
		if (counter == 2)
			temppoint = q;
		if (counter == 3)
			temppoint = p2;
		if (counter == 4)
			temppoint = q2;




		
		double x, y, r;
		int d1 = rand();
		x = (double)d1;
		x = x/32767.0;
		x = x*(2) - 1;
		d1 = rand();
		y = (double)d1;
		y = y/32767.0;
		y = y*2 - 1;
		r = (double)d1;
		r = r/32767.0;
		r = r*(0.1) + 0.02;


		vec = Point(x,y);
		vec.Norm();

		Point new_target_loc = temppoint + (wr + r)*vec;
		double new_target_rad = r;

		bool accept = true;
		for (int i = 0; i < num_targets; i++)
		{
			if (target_dims[i][2] == CIRCLE)
			{
				double len = (new_target_loc - target_locs[i]).Length() - new_target_rad - target_dims[i][0];
				if (len <= 0.02)
				{
					accept = false;
					i = 9099;
				}
				len = fabs(new_target_loc.x) + new_target_rad;
				if (len > 1.56)
					accept = false;
			}
		}

		double l1 = (new_target_loc - p).Length() - new_target_rad - wr;
		double l2 = (new_target_loc - q).Length() - new_target_rad - wr;
		if (l1 < 0.0 || l2 < 0.0)
			accept = false;

		l1 = (new_target_loc - p2).Length() - new_target_rad - wr;
		l2 = (new_target_loc - q2).Length() - new_target_rad - wr;
		if (l1 < 0.0 || l2 < 0.0)
			accept = false;

		if (accept)
		{
			target_locs[num_targets] = new_target_loc;
			target_dims[num_targets][0] = new_target_rad;
			target_dims[num_targets][1] = new_target_rad;
			target_dims[num_targets][2] = CIRCLE;
			num_targets++;
			counter++;
		}
	
	}

	Point new_target_loc;
	double new_target_rad;

	int count = 0;
	while (num_targets < 10 && count < 1000)
	{
		count ++;
		double x, y, r;
		int d1 = rand();
		x = (double)d1;
		x = x/32767.0;
		x = x*(d*scale*1.4) - d*scale*1.4/2;
		d1 = rand();
		y = (double)d1;
		y = y/32767.0;
		y = y*(2*w*2.8*scale) - w*2.8*scale;
		d1 = rand();
		r = (double)d1;
		r = r/32767.0;
		r = r*(0.1) + 0.02;

		new_target_loc = Point(x,y);
		new_target_rad = r;

		bool accept = true;
		for (int i = 0; i < num_targets; i++)
		{
			if (target_dims[i][2] == CIRCLE)
			{
				double len = (new_target_loc - target_locs[i]).Length() - new_target_rad - target_dims[i][0];
				if (len <= 0.08)
				{
					accept = false;
					i = 9099;
				}
				len = fabs(new_target_loc.x) + new_target_rad;
				if (len > 1.56)
					accept = false;
			}
		}


		double l1 = (new_target_loc - p).Length() - new_target_rad - wr;
		double l2 = (new_target_loc - q).Length() - new_target_rad - wr;
		if (l1 < 0.0 || l2 < 0.0)
			accept = false;

		l1 = (new_target_loc - p2).Length() - new_target_rad - wr;
		l2 = (new_target_loc - q2).Length() - new_target_rad - wr;
		if (l1 < 0.0 || l2 < 0.0)
			accept = false;

		if (accept)
		{
			target_locs[num_targets] = new_target_loc;
			target_dims[num_targets][0] = new_target_rad;
			target_dims[num_targets][1] = new_target_rad;
			target_dims[num_targets][2] = CIRCLE;
			num_targets++;
		}
	
	}




	flatten_tim = 1;



}

void CCubeView::InitMultiTargets()
{
	flatten_tim = 0;

	double scale = 0.003;

	double intermediates = fitts_int[condition_list[condition_num][0]];
	double width = fitts_w[condition_list[condition_num][1]]*scale;
	width = 24*scale;
	double bubble_width = fitts_we[condition_list[condition_num][2]]*width;
	double a;

	double d = bubble_width - width;
//	SAFETY_ZONE = (d/2);
//	if (SAFETY_ZONE > 0.05)
	SAFETY_ZONE = 0.05 + 16*0.003;


	
	

/*	if (rep_num == 0)
	{
		target_locs[0] = goal_pos;
		target_dims[0][0] = width/2;
		target_dims[0][1] = width/2;
		target_dims[0][2] = CIRCLE;
		num_targets = 1;
		current_target = 0;

		

		Point movement_vec = Point(1,0);
		Point obstacle_vec = movement_vec;

		obstacle_vec.Norm();
		obstacle_vec = (bubble_width)*obstacle_vec;

		Point obstacle_norm_vec;
		if (fabs(obstacle_vec.y > 0.001))
		{
			obstacle_norm_vec.x = 1;
			obstacle_norm_vec.y = (-1*obstacle_vec.x)/obstacle_vec.y;
		}
		else
		{
			obstacle_norm_vec.y = 1;
			obstacle_norm_vec.x = (-1*obstacle_vec.y)/obstacle_vec.x;
		}

		obstacle_norm_vec.Norm();
		obstacle_norm_vec = (bubble_width)*obstacle_norm_vec;


		Point p1, p2, p3, p4;


		p1 = goal_pos + obstacle_vec;
		p2 = goal_pos - obstacle_vec;
		p3 = goal_pos + obstacle_norm_vec;
		p4 = goal_pos - obstacle_norm_vec;


		target_locs[num_targets] = p1;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		num_targets++;
		target_locs[num_targets] = p2;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		num_targets++;
		target_locs[num_targets] = p3;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		num_targets++;
		target_locs[num_targets] = p4;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		num_targets++;
		num_selectable_targets = 5;
		return;

	}*/
	Point start_pos = goal_pos;



	bool accept = 0;
/*	while(!accept)
	{
		accept = 1;
		goal_pos = Point(GetRand(32*scale + 3*32*scale - BORDER_X,-32*scale - 3*32*scale + BORDER_X),GetRand(32*scale + 3*32*scale - BORDER_Y,-32*scale - 3*32*scale + BORDER_Y));
		a = (goal_pos - start_pos).Length();
		if (a/scale < 256)
			accept = 0;
	}*/
	if (sequence_num%2 == 0)
		goal_pos = target_sequence[0][rep_num-1];
	else
		goal_pos = target_sequence[1][rep_num-1];

	if (sequence_num == 3 || sequence_num == 4)
		goal_pos.x *= -1;
	else if (sequence_num == 5 || sequence_num == 6 )
		goal_pos.y *= -1;
	else if (sequence_num == 7 || sequence_num == 8)
	{
		goal_pos.x *= -1;
		goal_pos.y *= -1;

	}


	a = (goal_pos - start_pos).Length();
	TRACE("%2.2f\n",a/0.003);
	current_distance = a/0.003;
	current_distance += 0.5;
	current_distance = (int)current_distance;

	target_locs[0] = goal_pos;
	target_dims[0][0] = width/2;
	target_dims[0][1] = width/2;
	target_dims[0][2] = CIRCLE;
	num_targets = 1;

	Point movement_vec = goal_pos - start_pos;
	Point obstacle_vec = movement_vec;

	obstacle_vec.Norm();
	obstacle_vec = (bubble_width)*obstacle_vec;

	Point obstacle_norm_vec;
	if (fabs(obstacle_vec.y) > 0.001)
	{
		obstacle_norm_vec.x = 1;
		obstacle_norm_vec.y = (-1*obstacle_vec.x)/obstacle_vec.y;
	}
	else
	{
		obstacle_norm_vec.y = 1;
		obstacle_norm_vec.x = (-1*obstacle_vec.y)/obstacle_vec.x;
	}

	obstacle_norm_vec.Norm();
	obstacle_norm_vec = (bubble_width)*obstacle_norm_vec;


	Point p1, p2, p3, p4;


	p1 = goal_pos + obstacle_vec;
	p2 = goal_pos - obstacle_vec;
	p3 = goal_pos + obstacle_norm_vec;
	p4 = goal_pos - obstacle_norm_vec;


	target_locs[num_targets] = p1;
	target_dims[num_targets][0] = width/2;
	target_dims[num_targets][1] = width/2;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;
	target_locs[num_targets] = p2;
	target_dims[num_targets][0] = width/2;
	target_dims[num_targets][1] = width/2;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;
	target_locs[num_targets] = p3;
	target_dims[num_targets][0] = width/2;
	target_dims[num_targets][1] = width/2;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;
	target_locs[num_targets] = p4;
	target_dims[num_targets][0] = width/2;
	target_dims[num_targets][1] = width/2;
	target_dims[num_targets][2] = CIRCLE;
	num_targets++;


	double max_distractor_dist = (a - bubble_width - width);
	int max_num_distractors = max_distractor_dist/width;
	max_num_distractors *= intermediates;
	movement_vec.Norm();
	for (int i = 0; i < max_num_distractors; i++)
	{
		double dist = ((double)(i))*(p2-start_pos).Length()/((double)max_num_distractors);
		Point new_loc = start_pos + dist*movement_vec;
		if (i > 0)
		{
			obstacle_norm_vec.Norm();
			double theta = M_PI*((TIM_SECT_ANG_1/2)/180.0);
			double max_len = dist*atan(theta) + width/3;
			double offset_length = GetRand(-max_len,max_len);
			new_loc = new_loc + offset_length*obstacle_norm_vec;
		}
		target_locs[num_targets] = new_loc;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		num_targets++;
	}
	target_locs[num_targets] = start_pos;
	target_dims[num_targets][0] = width/2;
	target_dims[num_targets][1] = width/2;
	target_dims[num_targets][2] = CIRCLE;
	current_target = num_targets;
	num_targets++;




	int total_targets;
	if (intermediates == 0)
		total_targets = 20;
	else if (intermediates == 0.4)
		total_targets = 60;
	else total_targets = 100;

	double factor = (16*scale)/width;
	total_targets *= factor;

	

	int counter = 0;
	while (num_targets < total_targets && counter < 10000)
	{
		bool accept = true;
		counter++;
		Point new_target_loc = Point(GetRand(-BORDER_X+width/2,+BORDER_X-width/2),GetRand(-BORDER_Y+width/2,BORDER_Y-width/2));
		target_locs[num_targets] = new_target_loc;
		target_dims[num_targets][0] = width/2;
		target_dims[num_targets][1] = width/2;
		target_dims[num_targets][2] = CIRCLE;
		for (int i = 0; i < num_targets; i++)
		{
			if (target_dims[i][2] == CIRCLE)
			{
				double len = (new_target_loc - target_locs[i]).Length() - (width/2) - target_dims[i][0];
				if (len <= 0.02)
				{
					accept = false;
					i = 9099;
				}

			}
		}
		if (accept)
		{
			if (!TargetInsideSector(num_targets,start_pos,goal_pos-start_pos,TIM_SECT_ANG_1))
				num_targets++;
			else if ((new_target_loc-start_pos).Length() > (p1-start_pos).Length())
				num_targets++;
		}
		
	}







}
double CCubeView::GetDistance(Point start_pos, double width, double bubble_width)
{
	double min = 256;
	double max = (start_pos - Point(BORDER_X,BORDER_Y)).Length();
	if((start_pos - Point(-BORDER_X,BORDER_Y)).Length() > max)
		max = (start_pos - Point(-BORDER_X,BORDER_Y)).Length();
	if((start_pos - Point(-BORDER_X,-BORDER_Y)).Length() > max)
		max = (start_pos - Point(-BORDER_X,-BORDER_Y)).Length();
	if((start_pos - Point(BORDER_X,-BORDER_Y)).Length() > max)
		max = (start_pos - Point(BORDER_X,-BORDER_Y)).Length();
	max = max - width - bubble_width - 0.05;
	return GetRand(min,max/0.003);






}


void CCubeView::DrawTargets()
{

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

	for (int i = 0; i < num_targets; i++)
	{
		if (background == BLACK_BACKGROUND)
			glColor4f(0.4,0.4,0.4,0.5);
		else
			glColor4f(0.0,0.6,0.0,1.0);

		if (i == goal_target)
		{
			/*if (i == current_target)
			{
				if (background == BLACK_BACKGROUND)
					glColor4f(0.6,0.0,0.0,1.0);
				else
					glColor4f(0.8,0.0,0.0,1.0);
				if (toggle_color > 5)
					glColor4f(1.0,0.0,0.0,1.0);
			}
			else*/
			{
				if (background == BLACK_BACKGROUND)
					glColor4f(0.0,0.6,0.0,1.0);
				else
					glColor4f(0.8,0.0,0.0,1.0);
				if (toggle_color > 5)
					glColor4f(0.0,1.0,0.0,1.0);
			}

	//		toggle_color++;
	//		if (toggle_color == 10)
	//			toggle_color = 0;
		}
	//	if (i == select_target)
	//		glColor4f(1.0,0.7,0.0,1.0);

		glPushMatrix();
		glTranslatef(target_locs[i].x,target_locs[i].y,0);


		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
//		if (i != goal_target && i != current_target)
//			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

		glLineWidth(4.0);
		if (target_dims[i][2] == CIRCLE)
		{
			glBegin(GL_POLYGON);
			double r = target_dims[i][0];
			double a = 0;
			double seg_length = 2*M_PI;
			int loop = 1;
			double tot_rad = 0;
			while(1)
			{		
				double x = r*sin(a);
				double y = r*cos(a);
				glVertex3f(x,y,0);
				if (loop == 0)
					break;
				a += 0.04;
				tot_rad += 0.04;
				if (tot_rad >= seg_length)
				{
					a = 2*M_PI;
					loop = 0;
					break;
				}
			}
			glEnd();
		}
		else if (target_dims[i][2] == SQUARE)
		{
			double x = target_dims[i][0]/2;
			double y = target_dims[i][1]/2;

			glBegin(GL_POLYGON);
			glVertex3f(x,y,0.0);
			glVertex3f(x,-y,0.0);
			glVertex3f(-x,-y,0.0);
			glVertex3f(-x,y,0.0);
			glEnd();
		}


	/*	char numbuffer[5];
		sprintf(numbuffer,"%d\n",i);
		

		float s = 0.0006f;
		glPushMatrix();	
		glScalef( s, s, 1 );
		glColor4f(1.0,1.0,1.0,1.0);

		for ( int j = 0; numbuffer[j] != '\0'; ++j ) 
				glutStrokeCharacter( GLUT_STROKE_MONO_ROMAN, numbuffer[j] );
		glPopMatrix();*/




	
		glPopMatrix();
	}
}



void CCubeView::GetCircleCircleIntersection(Point p0, double r0, Point p1, double r1, Point *i1, Point *i2)
{
	double d = (p0 - p1).Length();
	double a = (r0*r0 - r1*r1 + d*d)/(2*d);
	double h = sqrt(r0*r0 - a*a);
	Point p2 = p0 + (a/d)*(p1-p0);

	i1->x = p2.x + h*(p1.y - p0.y)/d;
	i1->y = p2.y - h*(p1.x - p0.x)/d;

	i2->x = p2.x - h*(p1.y - p0.y)/d;
	i2->y = p2.y + h*(p1.x - p0.x)/d;


}



double CCubeView::GetCiricleSegmentLength(Point p0,double r0, Point i1, Point i2,Point cursor_loc)
{
	double circumf = 2*M_PI*r0;
	Point v1 = i1 - p0;
	Point v2 = i2 - p0;
	Point v3 = cursor_loc - p0;
	double a1 = v1.Ang(v3);
	double a2 = v3.Ang(v2);
	double a = v1.Ang(v2);
	double seg1 = circumf * a/(2*M_PI);
	if (fabs(a1 + a2 - a) < 0.001)
		seg1 = circumf - seg1;
//	TRACE("%2.2f\n",seg1);
	return seg1;

}

bool CCubeView::AngleInside(double a,double a1,double a2)
{
	if (a1 <= a2)
		return (a >= a1 && a <= a2);
	else
		return (a >= a1 || a <= a2);
}

void CCubeView::NextGoalTarget()
{
	
	while(1)
	{
		double i;
		int d1 = rand();
		i = (double)d1;
		i = i/32767.0;
		i = i*num_targets;
		int k = (int)i;
		if (k != goal_target)
		{
			goal_target = k;
			break;
		}
	}
//	goal_target = 0;
}		

void CCubeView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	char c = char(nChar); 



//	if (c == 'a' || c == 'A')
//	{
//		cursor = AREA;
//		cursor_rad = 0.2;
//		CalcCurrentTarget();
//	}
	if (c == 'b' || c == 'B')
	{
		cursor = BUBBLE;
		CalcCurrentTarget();
	}
	if (nChar == 27)
		AfxGetMainWnd()->PostMessage(WM_CLOSE);
	if (c == 'l' || c == 'L')
	{
		freeze = 1 - freeze;
		ShowCursor(freeze);
		if (freeze)
			MessageBox("Demo Paused, click L to resume", "Bubble Cursor");
	}


/*	if (c == 'p' || c == 'P')
	{
		cursor = POINT;
		CalcCurrentTarget();
	}*/
//	if (c == 'k' || c == 'K')
//		background = 1 - (background - 1) + 1;

	if (c == 'x' || c == 'X')
	{
		user_set_targets += 5;
		if (user_set_targets > 125)
			user_set_targets = 125;
		InitTargets();
		goal_target = 0;
	}
	if (c == 'z' || c == 'Z')
	{
		user_set_targets -= 5;
		if (user_set_targets < 5)
			user_set_targets = 5;
		InitTargets();
		goal_target = 0;
	}




//	if (c == 'L' || c == 'l')
//		lock_cursor = 1 - lock_cursor;


/*	if (c == 't' || c == 'T')
	{
		cursor = TIM;
		tim_loc = cursor_loc;
		InitTimLoc();
	}*/

//	if (c == 'c' || c == 'C')
//	{
//		TargetHit(0);
	/*	condition_num++;
		if (condition_num == 27)
			condition_num = 0;
		InitTargets(0,0);*/
	//	TargetHit(0);

/*	if (c == 'i' || c == 'I')
	{

		InitTargets();
		NextGoalTarget();
	}
*/

	if (c == 'q' || c == 'Q')
	{
		if (target_dims[0][2] == CIRCLE)
		{
			for (int i = 0; i < num_targets; i++)
			{
				target_dims[i][2] = SQUARE;
				target_dims[i][0] = target_dims[i][0]*2;
				target_dims[i][1] = target_dims[i][1]*2;
			}
		}
		else if (target_dims[0][2] == SQUARE)
		{
			for (int i = 0; i < num_targets; i++)
			{
				target_dims[i][2] = CIRCLE;
				target_dims[i][0] = target_dims[i][0]/2;
				target_dims[i][1] = target_dims[i][1]/2;
			}
		}

	}




	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CCubeView::InitTimLoc()
{






}





void CCubeView::CalcTimTarget()
{






}


int CCubeView::GetTimeOfDay(struct timeval2 *tp)
{    
	DWORD dw;
    dw = GetTickCount();    
	tp->tv_sec   = dw / 1000;
    tp->tv_usec  = (dw % 1000) * 1000;    
	return 1;
}

/* the time in seconds */
double CCubeView::GetSeconds()
{
  struct timeval2 t;
  GetTimeOfDay(&t);
  return ((double) t.tv_sec) + (((double) t.tv_usec)/1000000.0);
}



bool CCubeView::GetClosestTargetInsideSector(Point p,Point v, double angle)
{

return true;

}

static int testk;
 
bool CCubeView::TargetInsideSector(int target_num, Point p,Point v, double angle)
{
	
	return 0;
}


bool CCubeView::SectorsIntersect(Point v1, double a1, Point v2, double a2)
{
	double a = v1.Ang(v2);
//	TRACE("%d * %2.2f * %2.2f * %2.2f\n",testk,a1,a2,a);
	if (a > a1/2 + a2/2)
		return false;
	else
		return true;
}


void CCubeView::TargetHit(double t)
{
	NextGoalTarget();
	return;
	rep_num++;
	if (rep_num > 1) //we are ending a trial
	{
		final_click_time = t;
		final_click_point = cursor_loc;
		if (!warmup)
			RecordTrial();
	}
	else  //first rep of trial
		total_trial_start = t;




	if (rep_num > NUM_REPS)
	{
		num_conditions_complete++;
		if (num_conditions_complete == 27 || (warmup && num_conditions_complete == 6))
		{
			if (!warmup)
			{
				block_num++;
				if (block_num == NUM_BLOCKS)
				{
					if (do_another == 3)
					{
						dontdraw = 1;
						MessageBox("Thank You For Your Participation");
						exit(1);
					}
					else
					{
						cursor = cursor_order[do_another];
						do_another++;
						warmup = 1;
						dontdraw = 1;
						MessageBox("Click To Begin Warmup");
						dontdraw = 0;
						exp_line_num = 0;
						total_trials = 0;
						total_errors = 0;
						block_num = 0;
						num_conditions_complete = 0;
						NextBlock();
						InitMultiTargets();
					}
				}



				else
					NextBlock();
			}
			else
			{
				dontdraw = 1;
				MessageBox("Click To Begin Experiment");
				dontdraw = 0;
				warmup = 0;
				exp_line_num = 0;
				total_trials = 0;
				total_errors = 0;
				block_num = 0;
				num_conditions_complete = 0;
				NextBlock();
				InitMultiTargets();
			}





		}
		else
		{
			NextCondition();
			num_stamps = 0;
		}
	}
	else   ///we are starting a trial
	{
		delay_start_time = t;
		trial_start_time = -1;
		inside_start_time = -1;
		delay_start_loc = cursor_loc;
		error_flag = 0;
		total_trials++;
		StampData(t);
		InitMultiTargets();

	}
}


void CCubeView::RecordTrial()
{
	if (warmup)
		return;


	


	char cursor_char[50];
	if (cursor == BUBBLE)
		sprintf(cursor_char,"BUBBLE");
	else if (cursor == POINT)
		sprintf(cursor_char,"POINT");
	else if (cursor == TIM)
		sprintf(cursor_char,"TIM");

	if (error_flag == 0)
	{
		first_click_point = final_click_point;
		first_click_time = final_click_time;
	}


	double delay_time = trial_start_time - delay_start_time;
	double trial_time = final_click_time - trial_start_time;
	double first_trial_time = first_click_time - trial_start_time;
	double inside_time = final_click_time - inside_start_time;

	sprintf(trial_record[rep_num-2], "%d %d %d %d %s %3.6f %3.6f %3.6f %3.6f %d %2.2f %2.2f %2.2f %2.6f %2.6f %2.6f %2.6f %2.6f %2.6f %2.6f %d", SUBJECT_NUMBER, block_num, exp_line_num++, rep_num-2, cursor_char, first_trial_time, trial_time, delay_time, inside_time, condition_num, current_distance, fitts_int[condition_list[condition_num][0]], fitts_w[condition_list[condition_num][1]], fitts_we[condition_list[condition_num][2]]*fitts_w[condition_list[condition_num][1]], first_click_point.x, first_click_point.y, final_click_point.x, final_click_point.y, target_locs[goal_target].x,target_locs[goal_target].y, error_flag);




	if (rep_num == NUM_REPS+1)
	{
		exp_data.clear();
		exp_data.open(data_file, ios::app);
		while (!exp_data.is_open())
		{
			exp_data.clear();
			exp_data.open(data_file, ios::app);
		}

		for (int i = 0; i < NUM_REPS; i++)
			exp_data << trial_record[i] <<  "\n" << flush;

		exp_data.flush();
		exp_data.close();
		WritePointData(trial_time);

		char total_char[1000];
		sprintf(total_char, "%d %d %d %s %3.6f %d %2.2f %2.2f %2.6f", SUBJECT_NUMBER, block_num, total_line_num++, cursor_char, final_click_time - total_trial_start, condition_num, fitts_int[condition_list[condition_num][0]], fitts_w[condition_list[condition_num][1]], fitts_we[condition_list[condition_num][2]]*fitts_w[condition_list[condition_num][1]]);
		total_data.clear();
		total_data.open(total_file, ios::app);
		while (!total_data.is_open())
		{
			total_data.clear();
			total_data.open(data_file, ios::app);
		}
		total_data << total_char << "\n" << flush;
		total_data.flush();
		total_data.close();


	}
}

void CCubeView::WritePointData(double trial_time)
{

	exp_all_points.clear();
	exp_all_points.open(point_file, ios::app);
	while(!exp_all_points.is_open())
	{
		exp_all_points.clear();
		exp_all_points.open(point_file, ios::app);
	}



	exp_all_points << current_distance << " " << fitts_int[condition_list[condition_num][0]] << " " << fitts_w[condition_list[condition_num][1]] << " " << fitts_we[condition_list[condition_num][2]] << " ";
	for (int i = 0; i < num_targets; i++)
		exp_all_points << " " << target_locs[i].x << " " << target_locs[i].y << " " << target_dims[i][0];
	exp_all_points << "\n";


	for (int i = 0; i < num_stamps; i++)
	{
		exp_all_points << data_stamp[i][0] << " ";
		char timestamp[50];
		sprintf(timestamp,"%3.3f",data_stamp[i][1] - data_stamp[0][1]);
		exp_all_points << timestamp << " ";
		
		for (int j = 2; j < 5; j++)
			exp_all_points << data_stamp[i][j] << " ";
		exp_all_points << "\n" << flush;
	}
	exp_all_points.flush();
	exp_all_points.close();
}


void CCubeView::NextCondition()
{

	bool accept = false;
	while (!accept)
	{

		rand();
		rand();
		int d1 = rand();
		double x = (double)d1;
		x = x/32767.0;
		x = x*27;
		condition_num = (int)x;
		accept = true;
		if (condition_list[condition_num][3] == 1)
			accept = false;
		if (condition_num < 0)
			accept = false;
		if (condition_num > 26)
			accept = false;
	
	}
	rep_num = 0;
	condition_list[condition_num][3] = 1;
	goal_target = 0;
	sequence_num++;
	if (sequence_num == 8)
		sequence_num = 1;
	goal_pos = Point(0.0,0.0);
	InitMultiTargets();




}
void CCubeView::NextBlock()
{

	for (int i = 0; i < 27; i++)
		condition_list[i][3] = 0;//condition hasn't been done


	num_conditions_complete = 0;
	NextCondition();
}



void CCubeView::StampData(double t)
{

	if (warmup)
		return;
	data_stamp[num_stamps][0] = total_trials;
	data_stamp[num_stamps][1] = t;
	data_stamp[num_stamps][2] = cursor_loc.x;
	data_stamp[num_stamps][3] = cursor_loc.y;
	data_stamp[num_stamps][4] = lbutton;
	if (num_stamps < 1999)
		num_stamps++;
	last_stamp_time = t;
}

void CCubeView::TargetMiss(double t)
{
	if (trial_start_time == -1)
		return;

	if (rep_num >= 1)
	{
		if (error_flag == 0)
		{
			first_click_point = cursor_loc;
			first_click_time = t;
			error_flag = 1;
			total_errors++;
		}
		StampData(t);
		
	}
	oops = 1;
}

int CCubeView::GetCircleSquareIntersection(Point p0, double r0, Point p1, double w, double h, Point int_points[])
{
	int num_int_points = 0;
	Point p;
	Point vec;

	p = p1 + Point(w/2,0 - h/2);
	vec = Point(0,h);
	num_int_points = GetCircleSegmentIntersection(p0, r0, p, vec, int_points, num_int_points);

	p = p1 + Point(0 - w/2, 0 - h/2);
	vec = Point(0,h);
	num_int_points = GetCircleSegmentIntersection(p0, r0, p, vec, int_points, num_int_points);


	p = p1 + Point(0 - w/2, 0 - h/2);
	vec = Point(w,0);
	num_int_points = GetCircleSegmentIntersection(p0, r0, p, vec, int_points, num_int_points);


	p = p1 + Point(0 - w/2, 0 + h/2);
	vec = Point(w,0);
	num_int_points = GetCircleSegmentIntersection(p0, r0, p, vec, int_points, num_int_points);
	return num_int_points;




}

int CCubeView::GetCircleSegmentIntersection(Point p0, double r0, Point p, Point vec, Point int_points[], int num_int_points)
{
	Point p1 = p;
	Point p2 = p + vec;
	Point p3 = p0;


	double a = (p2.x - p1.x)*(p2.x - p1.x) + (p2.y-p1.y)*(p2.y-p1.y);
	double b = 2*( (p2.x-p1.x)*(p1.x-p3.x) + (p2.y-p1.y)*(p1.y-p3.y) );
	double c = p3.x*p3.x + p3.y*p3.y + p1.x*p1.x + p1.y*p1.y - 2*(p3.x*p1.x + p3.y*p1.y) - r0*r0;
	
	double t = b*b - 4*a*c;
	if (t < 0)
		return num_int_points;
	else if (fabs(t) < 0.00000000001)
	{
		///one int point
		double u = (-1*b)/(2*a);
		if (u < 0 || u > 1)
			return num_int_points;
		Point intp = p1 + u*vec;
		int_points[num_int_points] = intp;
		num_int_points++;
		return num_int_points;
	}
	else
	{
		//two int points
		double u = (-1*b + sqrt(b*b - 4*a*c))/(2*a);
		if (u >= 0 && u <= 1)
		{
			Point intp = p1 + u*vec;
			int_points[num_int_points] = intp;
			double testit = (intp - p0).Length();
			if (fabs(testit - r0) > 0.00001)
				int rsfhsehfks = 4;
			num_int_points++;
		}
		u = (-1*b - sqrt(b*b - 4*a*c))/(2*a);
		if (u >= 0 && u <= 1)
		{
			Point intp = p1 + u*vec;
			int_points[num_int_points] = intp;
			double testit = (intp - p0).Length();
			if (fabs(testit - r0) > 0.00001)
				int rsfhsehfks = 4;
			num_int_points++;
		}
		return num_int_points;
	}
}


double CCubeView::GetSquareSegmentLength(Point p0,double w, double h, Point i1, Point i2,Point cursor_loc)
{

	double circumf = 2*w + 2*h;

	double seg1 = 0;
	Point p = i1;
	while(!Infront(p, i2, p0, w, h))
	{
		Point q = NextCorner(p, p0, w, h);
		seg1 += (q - p).Length();
		p = q;
	}
	seg1 += (i2 - p).Length();


	Point i3 = i1 + 0.5*(i2 - i1);
	Point v1 = i1 - p0;
	Point v2 = i2 - p0;
	Point v3 = i3 - p0;
	double a1 = v1.Ang(v3);
	double a2 = v3.Ang(v2);
	double a = v1.Ang(v2);

	if (fabs(a1 + a2 - a) < 0.001)
	{
		seg1 = circumf - seg1;
		bubble_point1 = i2;
		bubble_point2 = i1;
	}
	else
	{
		bubble_point1 = i1;
		bubble_point2 = i2;
	}
	TRACE("%2.2f %2.2f\n",seg1, circumf);
	return seg1;



}

bool CCubeView::Infront(Point p, Point q, Point c, double w, double  h)
{
	double minx = c.x - (w/2);
	double maxx = c.x + (w/2);

	double miny = c.y - (h/2);
	double maxy = c.y + (h/2);

	int pside;
	int qside;

	if (fabs(p.x - maxx) < 0.0001 && p.y > miny)
		pside = 1;
	else if (fabs(p.y - miny) < 0.0001 && p.x > minx)
		pside = 2;
	else if (fabs(p.x - minx) < 0.0001 && p.y < maxy)
		pside = 3;
	else 
		pside = 4;

	if (fabs(q.x - maxx) < 0.0001 && q.y > miny)
		qside = 1;
	else if (fabs(q.y - miny) < 0.0001 && q.x > minx)
		qside = 2;
	else if (fabs(q.x - minx) < 0.0001 && q.y < maxy)
		qside = 3;
	else 
		qside = 4;

	if (pside != qside)
		return false;

	if (pside == 1)
		return (p.y >= q.y);
	else if (pside == 2)
		return (p.x >= q.x);

	else if (pside == 3)
		return (p.y <= q.y);

	else
		return (p.x <= q.x);
}


Point CCubeView::NextCorner(Point p, Point c, double  w, double h)
{
	double minx = c.x - (w/2);
	double maxx = c.x + (w/2);

	double miny = c.y - (h/2);
	double maxy = c.y + (h/2);


	Point q;




	if (fabs(p.x - maxx) < 0.0001 && p.y > miny)
		q = Point(maxx,miny);
	else if (fabs(p.y - miny) < 0.0001 && p.x > minx)
		q = Point(minx,miny);
	else if (fabs(p.x - minx) < 0.0001 && p.y < maxy)
		q = Point(minx,maxy);
	else 
		q = Point(maxx,maxy);


	return q;
}





void CCubeView::InitTargets(double w, double bw)
{

	num_targets = 0;
	w = fitts_w[condition_list[condition_num][1]];
	bw = fitts_we[condition_list[condition_num][2]];
	double d = bw*w;
	bw = w + d;
	SAFETY_ZONE = (d/2)*0.003;
	if (SAFETY_ZONE > 0.05)
		SAFETY_ZONE = 0.05;

//	d = fitts_a[condition_list[condition_num][0]];

	double scale = 0.003;
	w *= scale;
	bw *= scale;
	double borderx = BORDER_X;
	double bordery = BORDER_Y;
	double give = d*scale/4;
//	bw = bw + give;
	int num_horz = (2*(borderx))/bw;
	int num_vert = (2*(bordery))/bw;
	for (int i = 0; i <= num_horz && i <20; i++)
	{
		for (int j = 0; j <= num_vert && j < 20; j++)
		{
			Point p1 = Point(-1*borderx + (i)*bw, bordery - (j)*bw);

			double len = give;
			Point vec;
	
			double x, y;
			int d1 = rand();
			x = (double)d1;
			x = x/32767.0;
			x = x*(2) - 1;
			d1 = rand();
			y = (double)d1;
			y = y/32767.0;
			y = y*2 - 1;
	
			vec = Point(x,y);
			vec.Norm();
			vec = len*vec;
			Point p = p1 + vec;

			target_locs[num_targets] = p;
			target_dims[num_targets][0] = w/2;
			target_dims[num_targets][1] = w/2;//*new_target_rad;
			target_dims[num_targets][2] = CIRCLE;
			num_targets++;
		}
	}

	num_selectable_targets = num_targets;
	flatten_tim = 0;
}

Point CCubeView::GetSquareSegmentIntersection(Point c, double w, double h, Point p1, Point vec1)
{
	Point ip;

	Point p2;
	Point vec2;

	p2 = c + Point(w/2,0 - h/2);
	vec2 = Point(0,h);
	ip = GetSegmentSegmentIntersection(p1, vec1, p2, vec2);
	if (ip.x > -4)
		return ip;


	p2 = c + Point(0 - w/2, 0 - h/2);
	vec2 = Point(0,h);
	ip = GetSegmentSegmentIntersection(p1, vec1, p2, vec2);
	if (ip.x > -4)
		return ip;

	p2 = c + Point(0 - w/2, 0 - h/2);
	vec2 = Point(w,0);
	ip = GetSegmentSegmentIntersection(p1, vec1, p2, vec2);
	if (ip.x > -4)
		return ip;

	p2 = c + Point(0 - w/2, 0 + h/2);
	vec2 = Point(w,0);
	ip = GetSegmentSegmentIntersection(p1, vec1, p2, vec2);
	return ip;

}

Point CCubeView::GetSegmentSegmentIntersection(Point p1, Point vec1, Point p3, Point vec2)
{
	Point ret = Point(-5,-5);
	Point p2 = p1+vec1;
	Point p4 = p3+vec2;

	double denom = (p4.y-p3.y)*(p2.x-p1.x) - (p4.x-p3.x)*(p2.y-p1.y);
	if (denom == 0)
		return ret;
	double ua = ((p4.x-p3.x)*(p1.y-p3.y) - (p4.y-p3.y)*(p1.x-p3.x))/denom;
	double ub = ((p2.x-p1.x)*(p1.y-p3.y) - (p2.y-p1.y)*(p1.x-p3.x))/denom;
	if (ua <= 1 && ua >= 0 && ub <=1 && ub >= 0)
		ret = p1 + ua*vec1;
	return ret;
}


double CCubeView::GetRand(double min, double max)
{
	int d = rand();
	double ret = (double)d;
	ret = ret/32767.0;
	double b = min;
	double a = max - b;
	ret = a*ret + b;
	return ret;
}








void CCubeView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags, point);
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDblClk(nFlags, point);
}
