// cubeview.h : interface of the CCubeView class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.
#define BUBBLE 1
#define AREA 2
#define POINT 3
#define TIM 4

#define CIRCLE 1
#define SQUARE 2

#define TOTAL_TARGETS 25
#define VELOCITY_THRESHOLD 90
#define ACCELERATION_THRESHOLD -1000000


#define BORDER_X 1.55
#define BORDER_Y 1.05

#define NUM_TIM_POINTS 10
#define TIM_SECT_ANG_1 20 
#define TIM_SECT_ANG_2 25
#define TIM_SECT_ANG_3 30


#define NUM_BLOCKS 4
#define NUM_REPS 9


#define DAY_NUMBER 1
#define CURSOR_TYPE 3


#define WHITE_BACKGROUND 1
#define BLACK_BACKGROUND 2




#include "point.h"
#include <fstream>
using namespace std;


struct timeval2
{        
	long    tv_sec;         /* seconds */       
	long    tv_usec;        /* and microseconds */
};

class CCubeView : public CView
{
protected: // create from serialization only
	CCubeView();
	DECLARE_DYNCREATE(CCubeView)

// Attributes
public:
	CCubeDoc* GetDocument();


	int background;
	Point old_spot;
	Point new_spot;
	
	CPalette    m_cPalette;
	CPalette    *m_pOldPalette;
	CRect       m_oldRect;
	float       m_fRadius;
	float fAspect;
	int cursor;

	int cursor_order[3];
	bool oops;
	Point cursor_loc;
	double current_distance;
	double cursor_rad;
	double cursor_theta1;
	double cursor_theta2;
	double bubble_theta1;
	double bubble_theta2;
	double SAFETY_ZONE;
	int lock_cursor;
	Point mouse_loc, old_mouse_loc;
	bool cursor_showing;
	int select_target;
	int drag_target;
	Point drag_target_start;
	int toggle_color;
	int sequence_num;
	int SUBJECT_NUMBER;
	int do_another;
	int dontdraw;
	int total_line_num;
	int freeze;

	double inside_start_time;

	int rbutton;
	Point goal_pos;
	double next_dist;
	Point delay_start_loc;
	double delay_start_time;
	int user_set_targets;




	Point target_sequence[2][9];



	int lbutton;
	int current_target;
	Point target_locs[1000];
	double target_dims[1000][3];
	int num_targets;
	int num_selectable_targets;
	double fitts_w[18];
	double fitts_we[18];
	double fitts_int[18];
	int warmup;
	Point final_click_point;
	Point first_click_point;
	fstream exp_data;
	fstream total_data;
	double total_trial_start;
	double total_trial_end;
	double total_trial_time;
	double bubble_seg_length;

	double last_stamp_time;
	int flatten_tim;
	int fade_target;
	int fade_frame;
	Point tim_loc;
	double bubble_w, bubble_h;

	Point bubble_point1, bubble_point2;

	char trial_record[NUM_REPS][200];

	Point bubble_points[1000];
	int num_bubble_points;
	bool make_bubble;
	double bubble_rad;
	int bubble_frame;
	Point start_bubble_points[1000];
	Point final_bubble_points[1000];
	Point tim_points[NUM_TIM_POINTS];
	double tim_point_times[NUM_TIM_POINTS];
	Point mouse_points[NUM_TIM_POINTS];

	Point regress_vec;

	int goal_target;

	Point win_cent;

	Point test_point;
	Point test_vec;
	
	int exp_line_num;
	char data_file[60];
	char point_file[60];
	char total_file[60];

	int block_num;
	int rep_num;
	int condition_list[27][4];
	int condition_num;
	int num_conditions_complete;

	int error_flag;
	double first_click_time;
	double final_click_time;
	double trial_start_time;

	double total_trials;
	double total_errors;


	double data_stamp[2000][6];
	int num_stamps;


	fstream exp_all_points;

	CClientDC   *m_pDC;
	unsigned int backgroundList;



// Operations
public:
	void Init();
	void CreateRGBPalette(void);
	BOOL bSetupPixelFormat(void);
	unsigned char ComponentFromIndex(int i, UINT nbits, UINT shift);
	void DrawScene(void);
	void DrawCursor();
	void DrawBubbleCursor();
	void DrawPointCursor();
	void DrawAreaCursor();
	void DrawTimCursor();
	void CalcCursorRad();
	void InitTargets();
	void InitTargets(double w, double bw);
	void InitFittsTargets();
	void DrawTargets();
	void GetCircleCircleIntersection(Point p0, double r0, Point p1, double r1, Point *i1, Point *i2);
	int GetCircleSquareIntersection(Point p0, double r0, Point p1, double w, double h, Point int_points[]);
	double GetCiricleSegmentLength(Point p0,double r0, Point i1, Point i2,Point cursor_loc);
	double GetSquareSegmentLength(Point p0,double w, double h, Point i1, Point i2,Point cursor_loc);
	Point GetSquareSegmentIntersection(Point c, double w, double h, Point p1, Point vec1);
	Point GetSegmentSegmentIntersection(Point p1, Point vec1, Point p3, Point vec2);
	bool AngleInside(double a,double a1,double a2);
	void CreateBubblePoints(int closest_target);
	void NextGoalTarget();
	void CalcAreaTarget();
	void CalcPointTarget();
	void CalcCurrentTarget();
	void InitTimLoc();
	void CalcTimTarget();
	int LinRegress(double *x,double *y,int n,double *a,double *b,double *r);
	int GetTimeOfDay(struct timeval2 *tp);
	double GetSeconds();
	bool GetClosestTargetInsideSector(Point p,Point v, double angle);
	bool TargetInsideSector(int target_num, Point p,Point v, double angle);
	bool SectorsIntersect(Point v1, double a1,Point v2, double a2);
	int timVitesse();
	Point CalculEqDroite(Point newp, Point oldp);
	void NextCondition();
	void NextBlock();
	void TargetHit(double t);
	void StampData(double t);
	void WritePointData(double t1);
	void RecordTrial();
	void TargetMiss(double t);
	int GetCircleSegmentIntersection(Point p0, double r0, Point p, Point vec, Point int_points[], int num_int_points);
	double GetTheta(Point p1, double r, Point i1);
	bool Infront(Point p, Point q, Point c, double w, double  h);
	Point NextCorner(Point p, Point p0, double  w, double h);
	Point NextSquarePoint(Point p,  double inc, Point p0, double  w, double h);
	double GetDistance(Point start_pos, double width, double bubble_width);
	void InitMultiTargets();
	double GetRand(double min, double max);




// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCubeView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	//}}AFX_VIRTUAL

// Implementation
public:

	void InitLights();
	virtual ~CCubeView();
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);


#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	BOOL    m_play;

// Generated message map functions
protected:
	//{{AFX_MSG(CCubeView)
	afx_msg void OnFilePlay();
	afx_msg void OnUpdateFilePlay(CCmdUI* pCmdUI);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnUpdateAppCalibrate(CCmdUI* pCmdUI);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in cubeview.cpp
inline CCubeDoc* CCubeView::GetDocument()
   { return (CCubeDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
